package com.perforce.p4simulink;

/**
 * Copyright (C) 2014 Perforce Software. All rights reserved.
 *
 * Please see LICENSE.txt in top-level folder of this distribution.
 */

import java.awt.Frame;
import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.EnumSet;

import javax.swing.JOptionPane;

import com.mathworks.cmlink.api.ApplicationInteractor;
import com.mathworks.cmlink.api.ConfigurationManagementException;
import com.mathworks.cmlink.api.RepositorySupportedFeature;
import com.mathworks.cmlink.api.version.r14a.CMRepository;
import com.perforce.p4simulink.connection.Config;
import com.perforce.p4simulink.ui.P4ConnectionPane;
import com.perforce.p4simulink.util.Logging;

public class P4Repository extends P4Interactor implements CMRepository {

	private final int OK = 0;
	private final int CANCEL = 1;
	private final int ERROR = 2;

	private final Collection<RepositorySupportedFeature> supportedFeatures;

	public P4Repository(ApplicationInteractor applicationInteractor)
			throws P4CMException {
		this(applicationInteractor, new Config());
	}

	public P4Repository(ApplicationInteractor applicationInteractor,
			Config config) throws P4CMException {
		super(applicationInteractor, config);
		terminator = applicationInteractor.getTerminator();
		supportedFeatures = EnumSet
				.of(RepositorySupportedFeature.REPOSITORY_BROWSER);
	}

	@Override
	public void convertFolderToSandbox(File folder) {
		log.debug("convertFolderToSandbox() - {}", folder.getAbsolutePath());
		throw new UnsupportedOperationException();
	}

	@Override
	public boolean isFeatureSupported(RepositorySupportedFeature feature) {
		boolean supported = supportedFeatures.contains(feature);
		log.debug("isFeatureSupported() - {} {}", feature, supported);
		return supported;
	}

	@Override
	public void retrieveSandboxFromRepository(String repositorySpecifier,
			File sandboxRoot) throws ConfigurationManagementException {
		log.debug("retrieveSandboxFromRepository() {} {}", repositorySpecifier,
				sandboxRoot != null ? sandboxRoot.getAbsolutePath() : "");
		try {
			retrieveSandboxFromRepository(repositorySpecifier, sandboxRoot,
					null);
		} catch (ConfigurationManagementException e) {
			Logging.logException(log, e, true);
		}
	}

	public void retrieveSandboxFromRepository(String repositorySpecifier,
			File sandboxRoot, String tag)
			throws ConfigurationManagementException {
		log.debug("retrieveSandboxFromRepository() {} {} {}",
				repositorySpecifier,
				sandboxRoot != null ? sandboxRoot.getAbsolutePath() : "", tag);
		if (sandboxRoot == null
				|| (sandboxRoot != null && !sandboxRoot.exists())) {
			log.error("Check-out directory missing: "
					+ sandboxRoot.getAbsolutePath());
			return;
		}

		try {
			// here we need to get an adapter and check out the files in the
			// directory
			P4Adapter p4 = (P4Adapter) new P4AdapterFactory()
					.getAdapterForThisSandboxDir(sandboxRoot);
			p4.setDepotPath(repositorySpecifier);
			config = p4.config;
			p4.connect();
			if (!p4.login()) {
				showPasswordDialogue("");
				p4.login();
			}
			client = p4.getClient();
			Collection<File> files = Collections.singletonList(sandboxRoot);
			p4.checkout(files);
		} catch (ConfigurationManagementException e) {
			Logging.logException(log, e, true);
		}
	}

	public String getVersion() {
		log.debug("getVersion()");
		return "0.2.991735";
	}

	public Collection<String> getTags(String repositoryPath)
			throws ConfigurationManagementException {
		log.debug("getTags() - {}", repositoryPath);
		Collection<String> tags = new ArrayList<>();
		try {
			File repositoryFile = new File(repositoryPath);
			P4Adapter p4 = (P4Adapter) new P4AdapterFactory()
					.getAdapterForThisSandboxDir(repositoryFile);
			tags = p4.getTags(repositoryFile);
		} catch (ConfigurationManagementException e) {
			Logging.logException(log, e, true);
		}
		return tags;
	}

	@Override
	public String browseForRepositoryPath(String currentlySelectedPath,
			Frame parentFrame) {
		log.debug("browseForRepositoryPath() - {}", currentlySelectedPath);
		P4ConnectionPane pane = new P4ConnectionPane(config);
		if (depotPath == null || depotPath.isEmpty()) {
			depotPath = currentlySelectedPath;
		}
		boolean showDialogue = true;
		int tries = 5;
		int result = OK;
		while (showDialogue && tries-- > 0) {
			int option = JOptionPane.showConfirmDialog(
					applicationInteractor.getParentFrame(), pane.getFields(),
					"Open Connection", JOptionPane.OK_CANCEL_OPTION);

			if (option == JOptionPane.CANCEL_OPTION) {
				// user hit cancel
				result = CANCEL;
				break;
			}

			showDialogue = pane.getServer() == null
					|| pane.getServer().isEmpty() || pane.getUsername() == null
					|| pane.getUsername().isEmpty()
					|| pane.getWorkspace() == null
					|| pane.getWorkspace().isEmpty();
			result = !showDialogue ? OK : ERROR;
		}

		if (result == OK) {
			// we're good, so update the config values and save
			config.setPort(pane.getServer());
			config.setClient(pane.getWorkspace());
			config.setUser(pane.getUsername());

			try {
				config.save();
			} catch (P4CMException e) {
				log.error("Couldn't save config: " + e.getLocalizedMessage());
			}
		}

		return depotPath;
	}
}
